***************************** TI-85 SIMULATION LIBRARY
* ti85lib@0000 - initialization function, A5->8K buffer, A6->program
* ti85lib@0001 - "halt" instruction
* ti85lib@0002 - CLEARLCD call simulator
* ti85lib@0003 - OTH_PAUSE call simulator
* ti85lib@0004 - TX_CHARPUT call simulator
* ti85lib@0005 - D_ZT_STR call simulator
* ti85lib@0006 - GET_KEY call simulator
* ti85lib@0007 - "out" instruction, D1 = port number
* ti85lib@0008 - "in" instruction, D1 = port number
* ti85lib@0009 - FIND_PIXEL call simulator
* ti85lib@000A - LD_HL_MHL call simulator
* ti85lib@000B - D_LM_STR call simulator
* ti85lib@000C - M_CHARPUT call simulator
* ti85lib@000D - "ld a, r" instruction
* ti85lib@000E - D_ZM_STR call simulator
* ti85lib@000F - UNPACK_HL call simulator
* ti85lib@0010 - CP_HL_DE call simulator
* ti85lib@0011 - D_HL_DECI call simulator
* ti85lib@0012 - OTH_EXIT call simulator
* ti85lib@0013 - OTH_ARROW call simulator

        INCLUDE "tios.h"

        XDEF    _library
        XDEF    ti85lib@0000
        XDEF    ti85lib@0001
        XDEF    ti85lib@0002
        XDEF    ti85lib@0003
        XDEF    ti85lib@0004
        XDEF    ti85lib@0005
        XDEF    ti85lib@0006
        XDEF    ti85lib@0007
        XDEF    ti85lib@0008
        XDEF    ti85lib@0009
        XDEF    ti85lib@000A
        XDEF    ti85lib@000B
        XDEF    ti85lib@000C
        XDEF    ti85lib@000D
        XDEF    ti85lib@000E
        XDEF    ti85lib@000F
        XDEF    ti85lib@0010
        XDEF    ti85lib@0011
        XDEF    ti85lib@0012
        XDEF    ti85lib@0013

__ADDR  MACRO
    IFGT    \1-1
        move.w  d\1,d1
        __ADDR2 1
        move.l  a1,a\1
    ENDIF
    IFLE    \1-1
            __ADDR2 \1
    ENDIF
        ENDM

__ADDR2 MACRO
        sub.w   #$1000,d\1
        bclr    #15,d\1
        cmp.w   #$6000,d\1
        bge.s   ib\@
        move.l  prog_code(pc),a\1
        bra.s   ip\@
ib\@:   move.l  data_buffer(pc),a\1
ip\@:   add.w   d\1,a\1
        ENDM

***************************** INITIALIZATION

ti85lib@0000:
        move.l  a4,data_buffer          ; Save buffer address
        lea     $6000(a4),a5
        move.l  a6,prog_code
        move.l  sp,exit+2

        bsr     Clear_Screen

        move.l  #(8192/4)-1,d0          ; Clear buffer
clb_loop:
        move.l  d1,(a5)+
        dbra    d0,clb_loop

        move.w  #1,-(sp)                ; Set font to normal
        jsr     tios::FontSetSys
        addq.w  #2,sp

        move.w  #-6,d4                  ; Display warning message
        lea     warning+1(pc),a2
        bsr     Print_Messages

GetKey: move.l  keyboard,a3             ; Check for pressing '3'
        tst.w   (a3)                   
        beq.s   GetKey                
        clr.w   (a3)                
        move.w  2(a3),d0

        cmp.w   #51,d0
        beq.s   continue
        rts

continue:
        bsr     Clear_Screen            ; Clear the screen

        lea     $600000,a5              ; Install new interrupt
        bclr    #2,1(a5)
        move.l  $74,old_int5
        move.l  #new_int5,$74
        move.w  #$f9,$16(a5)

find_start:                             ; Find start of program & run it
        tst.b   (a6)+
        bne.s   find_start
        move.l  a6,d0
        btst    #0,d0
        beq.s   ok
        addq.w  #1,a6
ok:
        move.w  #$8346,d6               ; Initial value of IY
        move.w  #$400,d0                ; Disable low-level interrupts
        trap    #1

        tst.w   d7                      ; Push OTH_EXIT call if exact stack
        beq.s   noes
        move.w  d7,-(sp)
        jmp     (a6)

noes:   jsr     (a6)

ti85lib@0012:
exit:   lea     $12345678,a7
        moveq   #0,d0
        trap    #1
        move.w  #$b2,$600016            ; Restore interrupt
        move.l  old_int5,$74
        rts

***************************** NEW TIMER INTERRUPT

new_int5:
        move.l  a0,-(sp)              
        lea     count1(pc),a0
        add.w   #1,(a0)+                ; Inc/Test periodic action counters
        add.w   #1,(a0)
        cmp.w   #2,(a0)
        beq.s   update_keys
        cmp.w   #4,(a0)
        bne.s   done

        clr.w   (a0)+                   ; Redraw the screen
        tst.w   (a0)
        bne.s   done
        movem.l d0-d4/a1,-(sp)
        lea     LCD_MEM+16,a0
        move.l  data_buffer,a1
        lea     $6c00(a1),a1
        moveq   #63,d4
screen_loop:
        movem.l (a1)+,d0-d3
        movem.l d0-d3,-(a0)
        lea     46(a0),a0
        dbra    d4,screen_loop
        movem.l (sp)+,d0-d4/a1

done:   move.l  (sp)+,a0
        rte

update_keys:
        movem.l d0-d2/a1-a2,-(sp)

        lea     key_data_new(pc),a0     ; Save old keyboard data
        lea     key_data_old(pc),a1
        move.l  (a0)+,(a1)+
        move.l  (a0)+,(a1)+
        move.w  (a0)+,(a1)+

        lea     $600018,a0              ; Read raw keyboard data
        lea     key_data_new(pc),a1
        moveq   #$FFFFFFFE,d1
        moveq   #9,d0
read_key_loop:
        move.w  d1,(a0)
        rol.w   #1,d1
        nop
        nop
        move.b  3(a0),(a1)+
        dbra    d0,read_key_loop

        lea     key_data_new(pc),a0     ; Scan for newly pressed keys
        lea     key_data_chg(pc),a2
        moveq   #9,d1
key_scan_loop:
        move.b  (a0)+,d0
        not.b   d0
        and.b   (a1)+,d0
        move.b  d0,(a2)+
        dbra    d1,key_scan_loop

        moveq   #0,d1
        moveq   #$38,d2
        lea     keyconvtable_end,a0
get_key_loop:
        move.b  -(a0),d1                ; D0 = (bit << 4) | row
        and.b   #$f,d1
        move.b  key_data_chg(pc,d1.w),d0
        move.b  (a0),d1
        lsr.w   #4,d1
        btst    d1,d0
        bne.s   keys_done
        subq.w  #1,d2
        bne.s   get_key_loop

keys_done:
        move.b  d2,key_read
        movem.l (sp)+,d0-d2/a1-a2
        bra.s   done
        
key_data_new:
        dc.w    0,0,0,0,0
key_data_old:
        dc.w    0,0,0,0,0
key_data_chg:
        dc.w    0,0,0,0,0

key_read:
        dc.w    0

***************************** HALT INSTRUCTION

ti85lib@0001:
        lea     count1,a1
        clr.w   (a1)
wait:   tst.w   (a1)
        beq.s   wait
        rts

***************************** CLEARLCD CALL SIMULATOR

ti85lib@0002:
        move.l  data_buffer,a1
        lea     $6c00(a1),a1
        move.w  #255,d0
        moveq   #0,d1
clll:   move.l  d1,(a1)+
        dbra    d0,clll
        rts

***************************** OTH_PAUSE CALL SIMULATOR

ti85lib@0003:
        bsr     ti85lib@0006
        cmp.b   #13,d7
        bne.s   ti85lib@0003
        rts

***************************** TX_CHARPUT CALL SIMULATOR

string: dc.b    ' ',0

ti85lib@0004:
        move.w  #1,-(sp)                ; Set font to normal
        jsr     tios::FontSetSys
        addq.w  #2,sp

        move.l  data_buffer,a1      ; Load coordinates and check if onscreen
        moveq   #0,d0
        move.b  $700C(a1),d0
        cmp.w   #8,d0
        bge     off_screen
        moveq   #0,d1
        move.b  $700D(a1),d1
        cmp.w   #21,d1
        bge     off_screen

        addq.b  #1,$700D(A1)        ; Advance cursor coordinates
        cmp.b   #21,$700D(a1)
        bne.s   no_wrap
        clr.b   $700d(a1)
        addq.b  #1,$700c(a1)
no_wrap:

        move.w  d2,-(sp)
        st      stop                ; Screen display off

        asl.w   #3,d0               ; Calculate pixel coords and build stack
        mulu    #6,d1
        move.w  #4,-(sp)
        pea     string(pc)
        move.w  d0,-(sp)
        move.w  d1,-(sp)
        move.b  d7,string

        lea     $6c00(a1),a2        ; Calculate screen address (TI-85)
        asl.w   #4,d0
        add.w   d0,a2
        asr.w   #3,d1
        cmp.w   #15,d1
        bne.s   ok3
        moveq   #14,d1
ok3:
        add.w   d1,a2

        asr.w   #4,d0               ; Calculate screen address (TI-92)
        mulu    #30,d0
        lea     LCD_MEM,a3
        add.w   d1,a3
        add.w   d0,a3

        moveq   #7,d0               ; Copy TI-85 screen to TI-92 screen
loadscrn:
        move.b  (a2)+,(a3)+
        move.b  (a2)+,(a3)+
        lea     14(a2),a2
        lea     28(a3),a3
        dbra    d0,loadscrn

        jsr     tios::DrawStrXY     ; Call character drawing
        lea     10(sp),sp

        moveq   #7,d0               ; Copy TI-85 screen to TI-92 screen
savescrn:
        lea     -14(a2),a2
        lea     -28(a3),a3
        move.b  -(a3),-(a2)
        move.b  -(a3),-(a2)
        dbra    d0,savescrn

        sf      stop                ; Screen display on
        move.w  (sp)+,d2

off_screen:
        rts          

***************************** D_ZT_STR CALL SIMULATOR

ti85lib@0005:
        __ADDR  4
        addq.w  #1,d4
        move.b  (a4),d7
        beq.s   off_screen
        bsr     ti85lib@0004
        bra.s   ti85lib@0005

***************************** GET_KEY CALL SIMULATOR

ti85lib@0006:
        lea     key_read,a1
        move.b  (a1),d7
        clr.b   (a1)
        rts

***************************** OUT INSTRUCTION

ti85lib@0007:
        cmp.b   #1,d1
        beq.s   outport1
        rts
outport1:
        move.b  d7,keymask
        rts

***************************** IN INSTRUCTION

ti85lib@0008:
        cmp.b   #1,d1
        beq.s   inport1
        rts

inport1:
        lea     key_data_new(pc),a1
        movem.w d2-d4,-(sp)
        move.b  keymask,d0          ;
        moveq   #6,d1               ; D1 = 6 - row of TI-85 matrix
        moveq   #-1,d7              ; D7 = returned data
in1l:   asr.b   #1,d0               ; Test mask 
        bcs.s   noread
        bsr     read_row            ; If 0, read this row
noread: dbra    d1,in1l
        movem.w (sp)+,d2-d4
        rts

read_row:
        moveq   #6,d2
        sub.w   d1,d2
        asl.w   #3,d2               ; D2 = row of equiv. table
        lea     keyconvtable+8(pc,d2.w),a0
        moveq   #7,d3               ; D3 = bit in TI-85 matrix
loop_read:
        move.b  -(a0),d4             
        and.w   #$f,d4              ; D4 = row number
        move.b  0(a1,d4.w),d2       ; D2 = TI-92 key data
        move.b  (a0),d4
        lsr.w   #4,d4               ; D4 = bit number
        btst    d4,d2
        bne.s   notset
        bclr    d3,d7               ; If 0, clear TI-85 bit
notset: dbra    d3,loop_read
        rts

keymask:
        dc.w    -1

 ; Key equivalence table - Data from bits 0 to 7 in each TI-85 row,
 ; with row 0 first.  High nybble of each byte is the TI-92 bit, low
 ; nybble is the TI-92 row.

keyconvtable:
        dc.b    $70,$40,$60,$50,$78,$78,$78,$78
        dc.b    $19,$48,$09,$77,$05,$06,$57,$78
        dc.b    $79,$71,$72,$73,$64,$75,$67,$78
        dc.b    $69,$61,$62,$63,$54,$65,$30,$08
        dc.b    $59,$51,$52,$53,$32,$55,$21,$14
        dc.b    $38,$03,$74,$12,$56,$28,$24,$29
        dc.b    $47,$49,$42,$44,$46,$00,$68,$58
keyconvtable_end:
        dc.b    $78,$78,$78,$78,$78,$78,$78,$78

***************************** FIND_PIXEL CALL SIMULATOR

ti85lib@0009:
        move.b  d2,d4                   ; D4 (L) = Y coordinate
        move.w  d2,d0
        asr.w   #8,d0                   ; D0 = X coordinate
        move.b  d0,d1
        not.w   d1
        and.w   #7,d1                   ; D1 = bit number
        moveq   #0,d7
        bset    d1,d7                   ; D7 (A) = bit mask
        lsr.w   #3,d0                   ; D0 = X offset
        not.w   d4
        and.w   #$3f,d4                 ; D4 (HL) = real Y coordinate
        lsl.w   #4,d4                   ; D4 (HL) = Y offset
        add.w   d0,d4                   ; D4 (HL) = pixel offset
        and.w   #$3ff,d4
        rts

***************************** LD_HL_MHL CALL SIMULATOR

ti85lib@000A:
        __ADDR  4
        move.b  (a4)+,d0
        move.b  (a4),d4
        lsl.w   #8,d4
        move.b  d0,d4
        rts

***************************** D_LM_STR CALL SIMULATOR

ti85lib@000B:
        __ADDR  4
        move.w  d2,d0
        lsr.w   #8,d0
        subq.w  #1,d0
lmd:    move.b  (a4)+,d7
        move.w  d0,-(sp)
        bsr     ti85lib@000C
        move.w  (sp)+,d0
        dbra    d0,lmd
        rts

***************************** M_CHARPUT CALL SIMULATOR

ti85lib@000C:
        move.w  #0,-(sp)                ; Set font to small
        jsr     tios::FontSetSys
        addq.w  #2,sp

        move.l  data_buffer,a1      ; Load coordinates and check if onscreen
        moveq   #0,d0
        move.b  $7334(a1),d0        ; D0 = y
        cmp.w   #58,d0
        bge     off_screen2
        moveq   #0,d1
        move.b  $7333(a1),d1        ; D1 = x
        cmp.w   #124,d1
        bge     off_screen2

        addq.b  #4,$7333(A1)        ; Advance cursor coordinates

        move.w  d2,-(sp)
        st      stop                ; Screen display off

        move.w  #4,-(sp)            ; Build stack
        pea     string(pc)
        move.w  d0,-(sp)
        move.w  d1,-(sp)
        move.b  d7,string

        lea     $6c00(a1),a2        ; Calculate screen address (TI-85)

        lea     $6c00(a1),a2        ; Calculate screen address (TI-85)
        asl.w   #4,d0
        add.w   d0,a2
        asr.w   #3,d1
        cmp.w   #15,d1
        bne.s   ok2
        moveq   #14,d1
ok2:
        add.w   d1,a2

        asr.w   #4,d0               ; Calculate screen address (TI-92)
        mulu    #30,d0
        lea     LCD_MEM,a3
        add.w   d1,a3
        add.w   d0,a3

        moveq   #5,d0               ; Copy TI-85 screen to TI-92 screen
loadscrn2:
        move.b  (a2)+,(a3)+
        move.b  (a2)+,(a3)+
        lea     14(a2),a2
        lea     28(a3),a3
        dbra    d0,loadscrn2

        jsr     tios::DrawStrXY     ; Call character drawing
        lea     10(sp),sp

        moveq   #5,d0               ; Copy TI-85 screen to TI-92 screen
savescrn2:
        lea     -14(a2),a2
        lea     -28(a3),a3
        move.b  -(a3),-(a2)
        move.b  -(a3),-(a2)
        dbra    d0,savescrn2

        sf      stop                ; Screen display on
        move.w  (sp)+,d2

off_screen2:
        rts

***************************** LOAD A FROM R REGISTER

ti85lib@000D:
        move.w  d2,-(sp)
        jsr     tios::rand
        move.b  d0,d7
        move.w  (sp)+,d2
        rts

***************************** D_ZM_STR CALL SIMULATOR

ti85lib@000E:
        __ADDR  4
        addq.w  #1,d4
        move.b  (a4),d7
        beq.s   off_screen2
        bsr     ti85lib@000C
        bra.s   ti85lib@000E

***************************** UNPACK_HL CALL SIMULATOR

ti85lib@000F:
        ext.l   d4
        divu    #10,d4
        swap    d4
        move.b  d4,d7
        swap    d4
        rts

***************************** CP_HL_DE CALL SIMULATOR

ti85lib@0010:
        cmp.w   d3,d4
        rts

***************************** D_HL_DECI CALL SIMULATOR

format: dc.b    "%5u",0

ti85lib@0011:
        move.w  d2,-(sp)
        move.l  data_buffer(pc),a1
        move.w  d4,-(sp)
        pea     format(pc)
        pea     $70c6(a1)
        jsr     tios::sprintf
        lea     10(sp),sp
        move.w  #$80c6,d4
        move.w  (sp)+,d2
        jmp     ti85lib@0005

***************************** OTH_ARROW CALL SIMULAOTR

ti85lib@0013:
        move.b  #%10111110,keymask
        bra     inport1

***************************** MISC. ROUTINES
            
Clear_Screen:
        lea     $4440,a0                ; Clear the screen
        moveq   #0,d1                   
        move.l  #(3840/4)-1,d0
cls_loop:
        move.l  d1,(a0)+
        dbra    d0,cls_loop
        rts

Print_Messages:                         ; Display a sequence of strings
        moveq   #0,d1
        addq.w  #8,d4
        move.w  d4,d0
        lea     -1(a2),a0
        bsr     Display_String
loop_end_str:
        tst.b   (a2)+
        bne.s   loop_end_str
        tst.b   (a2)+
        bne.s   Print_Messages
        rts

Display_String:                         ; Display (A0) at (D1,D0)
        move.w  #4,-(sp)     
        move.l  a0,-(sp)
        move.w  d0,-(sp)
        move.w  d1,-(sp)
        jsr     tios::DrawStrXY
        lea     10(sp),sp
        rts

***************************** DATA

data_buffer:
        dc.l    0
prog_code:
        dc.l    0

keyboard:
        dc.l    tios::kb_vars+$1c

old_int5:
        dc.l    0

count1: dc.w    0
count2: dc.w    0
stop:   dc.w    0

warning:
        dc.b    '   WARNING: This program was',0
        dc.b    'automatically converted to',0
        dc.b    '68K code by Source Guru!',0

        dc.b    '   The conversion process is',0
        dc.b    'not completely reliable, so',0
        dc.b    'you should test the program',0
        dc.b    'thoroughly before deciding',0
        dc.b    'to trust it.  Refer to the',0
        dc.b    'Source Guru documentation',0
        dc.b    'for more information.',0

        dc.b    '   Press 3 to continue, or',0
        dc.b    'any other key to exit.',0,0

_library:
        dc.b    'ti85lib',0

        END
